/****************************************************************************
**
** Copyright (C) 2003-2004 Wolfpack.  All rights reserved.
** Copyright (C) 2004 Ralf Habacker
** Copyright (C) 2004 Andreas Hausladen
** Copyright (C) 2004,2005 Christian Ehrlicher
**
** This file is part of configure.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
**********************************************************************/

#include <windows.h>        // SearchPath()
#include <qnamespace.h>
#include <qregexp.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qstring.h>
#include <QTextStream>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>

// for Borland, main is defined to qMain which breaks configure
#undef main

#define DONT_SUPPORT_WINDOWSXP_STYLE 1

QTextStream out (stdout);

void usage( void );

struct Options
{
    QString install_prefix;
    QString QT_INSTALL_DOCS;
    QString QT_INSTALL_HEADERS;
    QString QT_INSTALL_LIBS;
    QString QT_INSTALL_BINS;
    QString QT_INSTALL_PLUGINS;
    QString QT_INSTALL_DATA;
    QString QT_INSTALL_TRANSLATIONS;
    QString QT_INSTALL_SYSCONF;

    QStringList QConfig;
    QStringList QCFlags;
    QStringList QModules;
    QStringList QNoModules;
    QStringList QStylesAvailable;
    QStringList QStyles;
    QStringList QStylesPlugin;
    QStringList QStylesNo;
    QStringList QImageFmtsAvailable;
    QStringList QImageFmts;
    QStringList QImageFmtsPlugin;
    QStringList QImageFmtsNo;
    QStringList sqlDriverAvailable;
    QStringList sqlDriver;
    QStringList sqlDriverPlugin;
    QStringList sqlDriverNo;
    QString qmakespec;
    bool build;
    bool verbose;
    bool fast;
    bool rebuild;
    bool makeFiles;
    bool dryrun;
    bool createDSPFiles;
    QString logfile;
}
options;

void my_debug( const QString &s )
{
    if ( options.verbose ) {
        printf( "[debug]: %s\n", s.toLatin1() );
    }
}

void my_system( const QString &s )
{
    my_debug ( s );
    if ( !options.dryrun )
        system( s.toLatin1() );
}

QString getMake()
{
    QString makespec = options.qmakespec;
    if ( makespec.startsWith( "win32-msvc" ) )
        return "nmake /NOLOGO";
    else if ( makespec == "win32-g++" )
        return "mingw32-make";
    else if ( makespec == "win32-borland" ) {
        char * fp, *buf = "";
        int len;
        /* use bmake if available, otherwise use make */
        len = SearchPathA ( NULL, "bmake.exe", NULL, 0, buf, &fp );
        if ( len )
            return "bmake";
    }
    return "make";
}

/* returns the correct Makefile* for building qmake... */
QString getMakefile()
{
    QString makespec = options.qmakespec;
    if ( makespec.startsWith( "win32-msvc" ) )
        return "Makefile.win32-msvc";
    if ( makespec == "win32-g++" )
        return "Makefile.win32-g++";
    else if ( makespec == "win32-borland" )
        return "Makefile.win32-borland";
    return "Makefile";
}

bool sameFile( const QString& f1, const QString& f2 )
{
    QFile file1( f1 );
    QFile file2( f2 );
    if ( !file1.open( QIODevice::ReadOnly ) || !file2.open( QIODevice::ReadOnly ) )
        return false;
    if ( file1.size() != file2.size() )
        return false;
    QByteArray b1 = file1.readAll();
    QByteArray b2 = file2.readAll();
    file1.close();
    file2.close();
    return ( b1 == b2 );
}

void buildqmake()
{
    QDir::setCurrent( options.install_prefix + "/qmake" );
    my_system( getMake() + " -f " + getMakefile() );
    QDir::setCurrent( options.install_prefix );
}


void callmake( const QString& folder )
{
    QString previous = QDir::current().absolutePath();
    QDir::setCurrent( folder );
    my_system( getMake() );
    QDir::setCurrent( previous );
}

void buildMakeSlow( const QString &file, QString &makefileName, QString &qmake_template,
                    bool createDSP )
{
    QString args = " -o " + makefileName;
    QString specarg = "-spec " + options.qmakespec;

    my_system( QDir::convertSeparators( options.install_prefix + "/bin/qmake " +
                                        specarg + " " + args + " " + file ) );

    if ( createDSP && !qmake_template.isEmpty() ) {
        args = "";
        my_system( QDir::convertSeparators( options.install_prefix + "/bin/qmake -t " +
                                            qmake_template + " " + specarg + " " +
                                            args + " " + file ) );
    }
}

void buildMakeFast( const QString &dir, const QString &file, QString &makefileName,
                    QString &qmake_template , bool )
{
    QString makefile = dir + "\\" + makefileName;
    QFile f( makefile );
    // fast configure - the makefile exists, skip it
    // since the makefile exists, it was generated by qmake, which means we
    // can skip it, since qmake has a rule to regenerate the makefile if the .pro
    // file changes...
    bool isBorlandMake = ( options.qmakespec == "win32-borland" );
    bool isMSVCMake = ( options.qmakespec.startsWith( "win32-msvc" ) );
    my_debug( "making " + makefile );

	if ( !f.open( QIODevice::WriteOnly | QIODevice::Truncate ) )
        qDebug( QString( "Error trying to open %1 for writing" ).arg( makefile ).toLatin1() );

    QTextStream s ( &f );

    s << "# " << makefile << ": generated by configure" << endl;
    s << "#" << endl;
    s << "# WARNING: This makefile will be replaced with a real makefile." << endl;
    s << "# All changes made to this file will be lost." << endl;
    s << endl;
    if ( isBorlandMake )
        s << "all: FORCE" << endl;
    else
        s << "all clean install qmake Makefile: FORCE" << endl;
    s << "\tcd " << QDir::convertSeparators(dir) << endl;
    s << "\t" << options.QT_INSTALL_BINS << "\\qmake -spec " << options.qmakespec << " -o " << makefileName << " " << file << endl;
    if ( isMSVCMake && !qmake_template.isEmpty() ) {
        s << "\t" << options.QT_INSTALL_BINS << "\\qmake -spec " << options.qmakespec << " -t " << qmake_template
        << " " << file << endl;
    }
    s << "\t$(MAKE)" << endl;
    s << endl;
    s << "FORCE:" << endl;
    if ( isBorlandMake )
        s << "\t@cd ." << endl;
    s << "\t" << endl;
    s << endl;

    if ( isBorlandMake ) {
        // other targets
        s << "clean: all" << endl;
        s << "install: all" << endl;
        s << "qmake: all" << endl;
        s << "Makefile: all" << endl;
    }
    s << endl;

    f.close();
}

void buildmake( const QString& dir, bool createDSP )
{
    QStringList excludeList = QString("bin config.tests cygwin debian doc include misc mkspecs").split(" ");
    QString makefileName;

    QDir d ( dir );
    if ( !d.exists() ) {
        out << "Can't read directory " << QDir::convertSeparators( dir );
        exit( 1 );
    }
	d.setFilter( QDir::AllDirs | QDir::Dirs | QDir::Files | QDir::Hidden | QDir::NoSymLinks );
    d.setNameFilters( QStringList("*.pro") );
    d.setSorting( QDir::Name );

    const QFileInfoList list = d.entryInfoList();

    for ( int i = 0; i < list.size(); ++i ) {
		QFileInfo fi = list.at(i);
        // skip "."  and ".." dir
        if ( fi.isDir() && ( fi.fileName() == "." || fi.fileName() == ".." ) )
            continue;
        else if ( excludeList.contains( fi.fileName() ) )
            continue;
        else if ( fi.isDir() )
            buildmake( d.absolutePath() + "/" + fi.fileName(), createDSP );
        else {
            QFile f( dir + "/" + fi.fileName() );
            if ( !f.open( QIODevice::ReadOnly ) )
                qDebug( qPrintable( QString( "Error trying to open %1 for reading" ).arg( f.fileName() ) ) );
            QString line;
            QString qmake_template;
            bool allow_fast = true;
            QTextStream s ( &f );
            while ( !s.atEnd() && !f.atEnd() && qmake_template.isEmpty() ) {
                line = s.readLine();
                if ( line.contains( "TEMPLATE" ) && line.toLower().contains( "app" ) )
                    qmake_template = "vcapp";
                else if ( line.contains( "TEMPLATE" ) && line.toLower().contains( "lib" ) ) {
                    qmake_template = "vclib";
                    allow_fast = false;
                } else if ( line.contains( "TEMPLATE" ) && line.toLower().contains( "subdirs" ) ) {
                    allow_fast = false;
                }
            }
            f.close();
            out << QString( "For %1\n" ).arg( QDir::convertSeparators( f.fileName() ) );
            makefileName = ( fi.fileName() == "qtmain.pro" ) ? "Makefile.main" : "Makefile";
            QString pwd = QDir::current().path();
            QDir::setCurrent( fi.path() );
            if ( options.fast && allow_fast ) {
                buildMakeFast( fi.path(), fi.fileName(), makefileName, qmake_template, createDSP );
            } else {
                buildMakeSlow ( fi.fileName(), makefileName, qmake_template, createDSP );
            }
            QDir::setCurrent( pwd );
        }
    }
}

QStringList getAvailableStyle()
{
	return QString("windows plastique windowsxp motif cde").split(" ");
}

QStringList getAvailableSqlDriver()
{
	QString driversDir = options.install_prefix + "/src/plugins/sqldrivers";
    QDir d( driversDir );
    if ( !d.exists() && !options.rebuild ) {
        if ( !options.makeFiles ) {
            out << "Can't read directory " << 
                     QDir::convertSeparators( driversDir ) << endl;
            exit( 1 );
        } else
            return QStringList();
    }

    const QFileInfoList list = d.entryInfoList();

    QStringList drivers;

    for ( int i = 0; i < list.size(); ++i ) {
		QFileInfo fi = list.at(i);
        if ( fi.isDir() && !fi.fileName().startsWith( "." ) && fi.fileName() != "CVS" && fi.fileName() != "tmp" )
            drivers.append( fi.fileName() );
    }
    return drivers;
}

static struct
{
    const char* option[ 2 ];
    int prefered;
}
config_struct1[] =
    { { "shared", "staticlib", 0 },
      { "incremental", "no-incremental", 1 },
      { "release", "debug", 0 },
      { "thread", "no-thread", 0 },
      { "largefile", "no-largefile", 1 },
      { "gif", "no-gif", 1 },
      { "tablet", "no-tablet", 1 },
      { "exceptions", "no-exceptions", 0 },
      { "rtti", "no-rtti", 0 },
      { "pch", "no-pch", 1 },
      { "big-codecs", "no-big-codecs", 0 },
      { NULL, NULL, 0 } };

/* this are normally much more (see qglobal.h) */
static struct  // option 2 has to be the "no-xxxx"
{
    const char* option[ 2 ];
    const char* define;
    int prefered;
    int remove; // remove 'no-xxx' from qconfig to reduce the qconfig output
}
config_struct2[] =
    { { "stl", "no-stl", "QT_NO_STL", 0, 1 },
      { "ipv6", "no-ipv6", "QT_NO_IPV6", 0, 1 },
      { "zlib", "no-zlib", "QT_NO_COMPRESS", 0, 0 },
      { "opengl", "no-opengl", "QT_NO_OPENGL", 1, 0 },
      { "sound", "no-sound", "QT_NO_SOUND", 0, 0 },
      { NULL, NULL, NULL, 0, 0 } };

/* Check for opposed config options and insert standard values if none set */
void CheckOptionsQConfig()
{
    int i;
    for ( i = 0; config_struct1[ i ].option[ 0 ]; i++ ) {
        int pref = config_struct1[ i ].prefered;
        if ( ( pref < 0 ) || ( pref > 2 ) )
            pref = 0;
        const char *option1 = config_struct1[ i ].option[ 0 ];
        const char *option2 = config_struct1[ i ].option[ 1 ];
        const char *prefStr = config_struct1[ i ].option[ pref ];

        /* check if both options are given */
        if ( options.QConfig.contains( option1 ) &&
                options.QConfig.contains( option2 ) ) {
            /* Remove the value which is prefered since the other was given by command line */
            options.QConfig.removeAll( prefStr );
        }
        /* check if none of the two options were given */
        if ( !options.QConfig.contains( option1 ) &&
                !options.QConfig.contains( option2 ) ) {
            /* Append standard option */
            options.QConfig.append( prefStr );
        }
    }
    for ( i = 0; config_struct2[ i ].option[ 0 ]; i++ ) {
        int pref = config_struct2[ i ].prefered;
        if ( ( pref < 0 ) || ( pref > 2 ) )
            pref = 0;
        const char *option1 = config_struct2[ i ].option[ 0 ];
        const char *option2 = config_struct2[ i ].option[ 1 ];
        const char *prefStr = config_struct2[ i ].option[ pref ];

        /* check if both options are given */
        if ( options.QConfig.contains( option1 ) &&
                options.QConfig.contains( option2 ) ) {
            /* Remove the value which is prefered since the other was given by command line */
            options.QConfig.removeAll( prefStr );
        }
        /* check if none of the two options were given */
        if ( !options.QConfig.contains( option1 ) &&
                !options.QConfig.contains( option2 ) ) {
            /* Append standard option */
            options.QConfig.append( prefStr );
        }
        if ( options.QConfig.contains( option2 ) ) {
            if ( config_struct2[ i ].remove )
                options.QConfig.removeAll ( option2 );
            options.QCFlags.append ( config_struct2[ i ].define );
        }
    }
    /* pch has to be precompile_header */
    if ( options.QConfig.contains( "pch" ) ) {
        options.QConfig.removeAll( "pch" );
        options.QConfig.append( "precompile_header" );
    }
    /* big-codecs is bigcodecs */
    if ( options.QConfig.contains( "big-codecs" ) ) {
        options.QConfig.removeAll( "big-codecs" );
        options.QConfig.append( "bigcodecs" );
    }
}

QString CheckDir( QString arg, QString type )
{
    QDir dir( arg );
    if ( !dir.exists() ) {
        out << QString( "WARNING: The specified path %1 for %2 doesn't exist" ).arg( QDir::convertSeparators ( dir.absolutePath() ) ).arg ( type ) << endl;
        exit ( 1 );
    }
    return ( QDir::convertSeparators( dir.absolutePath() ) );
}

void parseCommandLine( const QStringList & argv )
{
    // initialization
    options.install_prefix = getenv( "QTDIR" );
    if ( options.install_prefix.length() == 0 ) {
        out << "QTDIR not set!" << endl;
        exit ( 1 );
    }
    QDir qtdir( options.install_prefix );
    if ( !qtdir.exists() ) {
		out << "QTDIR " << options.install_prefix << " doesn't exist!";
        exit ( 1 );
    }

    // must be set before getAvailable ...()
    for ( int j = 1; j < argv.size(); ++j ) {
        QString arg( argv[ j ] );
        arg.remove( QRegExp( "^(-)*" ) );
        if ( arg == "make-files" )
            options.makeFiles = true;
    }

    options.install_prefix = qtdir.absolutePath();   // remove possible '\\' at end of path since this confuses qmake

    options.QModules = QString( "styles tools thread kernel widgets dialogs iconview workspace network canvas table xml opengl sql accessibility tablet sound" ).split( " " );
	options.QNoModules = QString( "" ).split(" ");
	options.QConfig = QString( "enterprise nocrosscompiler stl rtti pch warn_off create_prl link_prl minimal-config small-config medium-config large-config full-config" ).split(" ");
    options.QStyles = options.QStylesAvailable = getAvailableStyle();
	options.QImageFmts = options.QImageFmtsAvailable = QString( "png mng jpeg" ).split(" ");
    options.sqlDriverAvailable = getAvailableSqlDriver();
    options.build = true;

    QString configStatus = "";

    options.qmakespec = QString( getenv( "QMAKESPEC" ) ).toLower();

    // Enabled by default when using compilers that use Visual Studio Project files ;)
    if ( options.qmakespec.startsWith( "win32-msvc" ) || options.qmakespec == "win32-icc" )
        options.createDSPFiles = true;
    else
        options.createDSPFiles = false;

    for ( int i = 1; i < argv.size(); ++i ) {
        configStatus += argv[ i ] + QString( " " );
        QString arg( argv[ i ] );
        QString tmp;
        arg.remove( QRegExp( "^(-)*" ) );
        my_debug( arg );

        if ( arg == "help" ) {
            usage();
            exit( 1 );
        } else if ( arg == "v" || arg == "verbose" ) {
            options.verbose = true;
        } else if ( arg == "fast" ) {
            options.fast = true;
        } else if ( arg == "rebuild" ) {
            options.rebuild = true;
        } else if ( arg == "dryrun" || arg == "n" ) {
            options.dryrun = true;
        } else if ( arg == "no-dsp" ) {
            options.createDSPFiles = false;
        } else if ( arg == "dsp" ) {
            options.createDSPFiles = true;
            /* Styles */
        } else if ( arg.startsWith( "qt-style-" ) ) {
            tmp = arg.remove( "qt-style-" ).trimmed();
            if ( !options.QStylesAvailable.contains( tmp ) ) {
                qWarning( "Specified style %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.QStyles.append( tmp );
            options.QStylesNo.removeAll( tmp );
            options.QStylesPlugin.removeAll( tmp );
        } else if ( arg.startsWith( "plugin-style-" ) ) {
            tmp = arg.remove( "plugin-style-" ).trimmed();
            if ( !options.QStylesAvailable.contains( tmp ) ) {
                qWarning( "Specified style %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.QStyles.removeAll( tmp );
            options.QStylesNo.removeAll( tmp );
            options.QStylesPlugin.append( tmp );
        } else if ( arg.startsWith( "no-style-" ) ) {
            tmp = arg.remove( "no-style-" ).trimmed();
            if ( !options.QStylesAvailable.contains( tmp ) ) {
                qWarning( "Specified style %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.QStyles.removeAll( tmp );
            options.QStylesNo.append( tmp );
            options.QStylesPlugin.removeAll( tmp );
            /* Imageformats */
        } else if ( arg.startsWith( "qt-imgfmt-" ) ) {
            tmp = arg.remove( "qt-imgfmt-" ).trimmed();
            if ( !options.QImageFmtsAvailable.contains( tmp ) ) {
                qWarning( "Specified imageformat %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.QImageFmts.append( tmp );
            options.QImageFmtsNo.removeAll( tmp );
            options.QImageFmtsPlugin.removeAll( tmp );
        } else if ( arg.startsWith( "plugin-imgfmt-" ) ) {
            tmp = arg.remove( "plugin-imgfmt-" ).trimmed();
            if ( !options.QImageFmtsAvailable.contains( tmp ) ) {
                qWarning( "Specified imageformat %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.QImageFmts.removeAll( tmp );
            options.QImageFmtsNo.removeAll( tmp );
            options.QImageFmtsPlugin.append( tmp );
        } else if ( arg.startsWith( "no-imgfmt-" ) ) {
            tmp = arg.remove( "no-imgfmt-" ).trimmed();
            if ( !options.QImageFmtsAvailable.contains( tmp ) ) {
                qWarning( "Specified imageformat %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.QImageFmts.removeAll( tmp );
            options.QImageFmtsNo.append( tmp );
            options.QImageFmtsPlugin.removeAll( tmp );
            /* Modules */
        } else if ( arg.startsWith( "disable-" ) ) {
            tmp = arg.remove( "disable-" ).trimmed();
            options.QModules.removeAll( tmp );
            options.QNoModules.append( tmp );
            /* SQL */
        } else if ( arg.startsWith( "qt-sql-" ) ) {
            tmp = arg.remove( "qt-sql-" ).trimmed();
            if ( !options.sqlDriverAvailable.contains( tmp ) ) {
                qWarning( "Specified sql-driver %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.sqlDriver.append( tmp );
            options.sqlDriverNo.removeAll( tmp );
            options.sqlDriverPlugin.removeAll( tmp );
        } else if ( arg.startsWith( "plugin-sql-" ) ) {
            tmp = arg.remove( "plugin-sql-" ).trimmed();
            if ( !options.sqlDriverAvailable.contains( tmp ) ) {
                qWarning( "Specified sql-driver %s doesn't exist", qPrintable( tmp ) );
                exit ( 1 );
            }
            options.sqlDriver.removeAll( tmp );
            options.sqlDriverNo.removeAll( tmp );
            options.sqlDriverPlugin.append( tmp );
        } else if ( arg.startsWith( "no-sql-" ) ) {
            tmp = arg.remove( "no-sql-" ).trimmed();
            if ( !options.sqlDriverAvailable.contains( tmp ) ) {
                qWarning( "Specified sql-driver %s doesn't exist", tmp.toLatin1() );
                exit ( 1 );
            }
            options.sqlDriver.removeAll( tmp );
            options.sqlDriverNo.append( tmp );
            options.sqlDriverPlugin.removeAll( tmp );
        } else if ( arg == "prefix" ) {
            options.install_prefix = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "bindir" ) {
            options.QT_INSTALL_BINS = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "libdir" ) {
            options.QT_INSTALL_LIBS = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "headerdir" ) {
            options.QT_INSTALL_HEADERS = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "docdir" ) {
            options.QT_INSTALL_DOCS = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "plugindir" ) {
            options.QT_INSTALL_PLUGINS = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "datadir" ) {
            options.QT_INSTALL_DATA = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "translationdir" ) {
            options.QT_INSTALL_TRANSLATIONS = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "sysconfdir" ) {
            options.QT_INSTALL_SYSCONF = CheckDir ( argv[ ++i ], arg );
        } else if ( arg == "release" || arg == "debug" || arg == "shared" ||
                    arg == "incremental" || arg == "no-incremental" ||
                    arg == "thread" || arg == "no-thread" || arg == "largefile" || arg == "no-largefile" ||
                    arg == "gif" || arg == "no-gif" ||
                    arg == "zlib" || arg == "no-zlib" || arg == "opengl" || arg == "no-opengl" ||
                    arg == "tablet" || arg == "no-tablet" || arg == "exceptions" || arg == "no-exceptions" ||
                    arg == "rtti" || arg == "no-rtti" || arg == "stl" || arg == "no-stl" ||
                    arg == "pch" || arg == "no-pch" || arg == "ipv6" || arg == "no-ipv6" ||
                    arg == "sound" || arg == "no-sound" || arg == "big-codecs" || arg == "no-big-codecs" )
            options.QConfig.append( arg );
        else if ( arg == "static" )
            options.QConfig.append( "staticlib" );
        else if ( arg == "spec" )
            options.qmakespec = argv[ ++i ];
        else if ( arg == "log" ) {
            options.logfile = argv[ ++i ];
        } else if ( arg == "build" ) {
            options.build = true;
        } else if ( arg == "nobuild" ) {
            options.build = false;
        } else {
            qWarning( qPrintable( QString( "Ops, unrecognized option [%1]\n" ).arg( arg ) ) );
			usage();
            exit ( 1 );
        }
    }
    CheckOptionsQConfig();
}

void deletePlugins( QString dir )
{
    QDir d ( dir );
    if ( !d.exists() ) {
        out << "Can't read directory " << QDir::convertSeparators( dir );
        exit( 1 );
    }
	d.setFilter( QDir::AllDirs | QDir::Dirs | QDir::Files | QDir::Hidden | QDir::NoSymLinks );
    d.setNameFilters( QString("*.dll *.lib *.exp *.pdb").split( " " ) );
    d.setSorting( QDir::Name );

    const QFileInfoList list = d.entryInfoList();

    for ( int i = 0; i < list.size(); ++i ) {
		QFileInfo fi = list.at( i );
        // skip "."  and ".." dir
        if ( fi.isDir() && ( fi.fileName() == "." || fi.fileName() == ".." ) )
            continue;
        else if ( fi.isDir() )
            deletePlugins( d.absolutePath() + "/" + fi.fileName() );
        else {
            QFile f( dir + "/" + fi.fileName() );
            QString str = "deleting " + dir + "\\" + fi.fileName();
            my_debug( str );
            f.remove();
        }
    }
}

void doBuild()
{
    out << "Creating makefiles. Please wait...\n";
    buildmake( options.install_prefix, options.createDSPFiles );

    if ( options.build ) {
        out << "Building qt..." << endl;
        callmake( options.install_prefix );
    }
}

void makeFiles()
{
    QString buildDir = QDir::current().absolutePath();

    out << "Creating makefiles. Please wait...\n";
    buildmake( buildDir, options.createDSPFiles );

    out << "Building package ...\n";
    callmake( buildDir );
}

void WriteUnusedImageformats( QTextStream &s, QString format )
{
    if ( options.QConfig.contains( "no-" + format.toLower() ) ) {
        s << "#ifndef QT_NO_IMAGEIO_" << format.toUpper() << endl;
        s << "# define QT_NO_IMAGEIO_" << format.toUpper() << endl;
        s << "#endif\n\n";
    }
}

void doConfigure()
{
    //#-------------------------------------------------------------------------------
    //# post initialize QT_INSTALL_* variables, and generate qconfig.cpp
    //#-------------------------------------------------------------------------------

    options.install_prefix = QDir::convertSeparators( options.install_prefix );
    if ( options.QT_INSTALL_DOCS.isEmpty() )
        options.QT_INSTALL_DOCS = QDir::convertSeparators( options.install_prefix + "/doc" );
    if ( options.QT_INSTALL_HEADERS.isEmpty() )
        options.QT_INSTALL_HEADERS = QDir::convertSeparators( options.install_prefix + "/include" );
    if ( options.QT_INSTALL_LIBS.isEmpty() )
        options.QT_INSTALL_LIBS = QDir::convertSeparators( options.install_prefix + "/lib" );
    if ( options.QT_INSTALL_BINS.isEmpty() )
        options.QT_INSTALL_BINS = QDir::convertSeparators( options.install_prefix + "/bin" );
    if ( options.QT_INSTALL_PLUGINS.isEmpty() )
        options.QT_INSTALL_PLUGINS = QDir::convertSeparators( options.install_prefix + "/plugins" );
    if ( options.QT_INSTALL_DATA.isEmpty() )
        options.QT_INSTALL_DATA = QDir::convertSeparators( options.install_prefix );
    if ( options.QT_INSTALL_TRANSLATIONS.isEmpty() )
        options.QT_INSTALL_TRANSLATIONS = QDir::convertSeparators( options.install_prefix + "/translations" );
    if ( options.QT_INSTALL_SYSCONF.isEmpty() )
        options.QT_INSTALL_SYSCONF = QDir::convertSeparators( options.install_prefix );

    // Give feedback

#define HasConfig(x)   QString ( options.QConfig.contains(x) && !options.QConfig.contains("no-"x) ? "yes" : "no" ) + "\n"
#define HasFlag(x)     QString ( options.x ? "yes" : "no" ) + "\n"
#define HasImageFmt(x) QString ( options.QImageFmtsNo.contains(x) ? "no" : "yes" ) + "\n"

#define ifConfig(x) (options.QConfig.contains(x) && !options.QConfig.contains("no-"x) ? "yes" : "no")

    out << "Incremental ......... " << HasConfig( "incremental" );
    out << "Configuration ....... " << options.QConfig.join( " " ) << endl;
    out << "STL support ......... " << HasConfig( "stl" );
    out << "PCH support ......... " << HasConfig( "precompile_header" );
    out << "IPv6 support ........ " << HasConfig( "ipv6" );
    out << "Thread support ...... " << HasConfig( "thread" );
    out << "Debug symbols........ " << HasConfig( "debug" );
    out << "verbose mode......... " << HasFlag( verbose );
    out << "fast mode............ " << HasFlag( fast );

    out << "Large File support .. " << HasConfig( "largefile" );
    out << "QMAKESPEC............ " << options.qmakespec << endl;

	out << "GIF support ......... " << HasConfig( "gif" );
    out << "PNG support ......... " << HasImageFmt( "png" );
    out << "MNG support ......... " << HasImageFmt( "mng" );
    out << "JPEG support ........ " << HasImageFmt( "jpeg" );
    out << "zlib support ........ " << HasConfig( "zlib" );
    out << endl;
    out << "OpenGL support ...... " << HasConfig( "opengl" );
    out << "Tablet support ...... " << HasConfig( "tablet" );
    out << "Exception support.... " << HasConfig( "exceptions" );
    out << "RTTI support......... " << HasConfig( "rtti" );
    out << "SOUND support ....... " << HasConfig( "sound" );
    out << endl;
    out << "SQL driver  ......... " << options.sqlDriver.join( " " ) << endl;
    out << "SQL driver plugins .. " << options.sqlDriverPlugin.join( " " ) << endl;
    out << endl;
    out << "Styles .............. " << options.QStyles.join( " " ) << endl;
    out << "Style plugins ....... " << options.QStylesPlugin.join( " " ) << endl;
    out << endl;
    out << "Install prefix....... " << options.install_prefix << endl;
    out << "Headers path......... " << options.QT_INSTALL_HEADERS << endl;
    out << "Libraries path....... " << options.QT_INSTALL_LIBS << endl;
    out << "Binaries path........ " << options.QT_INSTALL_BINS << endl;
    out << "Plugins path......... " << options.QT_INSTALL_PLUGINS << endl;

    if ( options.QConfig.contains( "staticlib" ) )
        out << endl <<
                 "WARNING: Using static linking will disable the use of plugins."
                 "Make sure you compile ALL needed modules into the library."
                 << endl ;

#undef HasConfig

    out << endl << endl << endl;

    // # generate qconfig.cpp
	QFile f( options.install_prefix + "/src/corelib/global/qconfig.cpp.new" );
    if ( f.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
		QString qtdir = QDir::convertSeparators( options.install_prefix );
		if ( qtdir.endsWith("\\") )
			qtdir.chop( 1 );
		qtdir = qtdir.replace( "\\", "\\\\" );
        out << "generating qconfig.cpp" << endl;
        QTextStream s( &f );
		s << "/* Licensed */\n" <<
		"static const char qt_configure_licensee_str          [512 + 12] = \"qt_lcnsuser=Open Source\";\n"
		"static const char qt_configure_licensed_products_str [512 + 12] = \"qt_lcnsprod=OpenSourceDesktop\";\n";
		s << QString ( 
		"static const char qt_configure_prefix_path_str       [512 + 12] = \"qt_prfxpath=%1\";\n"
		"static const char qt_configure_documentation_path_str[512 + 12] = \"qt_docspath=%1\\\\doc\";\n"
		"static const char qt_configure_headers_path_str      [512 + 12] = \"qt_hdrspath=%1\\\\include\";\n"
		"static const char qt_configure_libraries_path_str    [512 + 12] = \"qt_libspath=%1\\\\lib\";\n"
		"static const char qt_configure_binaries_path_str     [512 + 12] = \"qt_binspath=%1\\\\bin\";\n"
		"static const char qt_configure_plugins_path_str      [512 + 12] = \"qt_plugpath=%1\\\\plugins\";\n"
		"static const char qt_configure_data_path_str         [512 + 12] = \"qt_datapath=%1\";\n"
		"static const char qt_configure_translations_path_str [512 + 12] = \"qt_trnspath=%1\\\\translations\";\n"
		"static const char qt_configure_examples_path_str     [512 + 12] = \"qt_xmplpath=%1\\\\examples\";\n"
		"static const char qt_configure_demos_path_str        [512 + 12] = \"qt_demopath=%1\\\\demos\";\n" )
		.arg( qtdir ) <<
		"/* strlen( \"qt_lcnsxxxx\" ) == 12 */\n"
		"#define QT_CONFIGURE_LICENSEE qt_configure_licensee_str + 12;\n"
		"#define QT_CONFIGURE_LICENSED_PRODUCTS qt_configure_licensed_products_str + 12;\n"
		"#define QT_CONFIGURE_PREFIX_PATH qt_configure_prefix_path_str + 12;\n"
		"#define QT_CONFIGURE_DOCUMENTATION_PATH qt_configure_documentation_path_str + 12;\n"
		"#define QT_CONFIGURE_HEADERS_PATH qt_configure_headers_path_str + 12;\n"
		"#define QT_CONFIGURE_LIBRARIES_PATH qt_configure_libraries_path_str + 12;\n"
		"#define QT_CONFIGURE_BINARIES_PATH qt_configure_binaries_path_str + 12;\n"
		"#define QT_CONFIGURE_PLUGINS_PATH qt_configure_plugins_path_str + 12;\n"
		"#define QT_CONFIGURE_DATA_PATH qt_configure_data_path_str + 12;\n"
		"#define QT_CONFIGURE_TRANSLATIONS_PATH qt_configure_translations_path_str + 12;\n"
		"#define QT_CONFIGURE_EXAMPLES_PATH qt_configure_examples_path_str + 12;\n"
		"#define QT_CONFIGURE_DEMOS_PATH qt_configure_demos_path_str + 12;\n";
	}
    f.close();

    // Avoid rebuilds checking if it didn't change.
    QDir qtInstallPrefixDir ( options.install_prefix );
    if ( !sameFile( options.install_prefix + "/src/corelib/global/qconfig.cpp.new", options.install_prefix + "/src/corelib/global/qconfig.cpp" ) ) {
        out << "activating qconfig.cpp" << endl;
        QFile::remove( options.install_prefix + "/src/corelib/global/qconfig.cpp" );
        qtInstallPrefixDir.rename( options.install_prefix + "/src/corelib/global/qconfig.cpp.new", options.install_prefix + "/src/corelib/global/qconfig.cpp" );
        // TODO: check if original file is readonly, in this case the config isn't updated
    } else
        QFile::remove( options.install_prefix + "/src/corelib/global/qconfig.cpp.new" );

    //# build up the variables for output
    //#-------------------------------------------------------------------------------
    //# part of configuration information goes into qmodules.h
    //#-------------------------------------------------------------------------------

    f.setFileName( options.install_prefix + "/include/qmodules.h.new" );
    if ( f.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
        QTextStream s( &f );
        for ( int i = 0; i < options.QModules.count(); ++i ) {
            s << "#define QT_MODULE_" << options.QModules[ i ].toUpper() << endl;
        }
        f.close();
    }

    // Avoid rebuilds checking if it didn't change.
    if ( !sameFile( options.install_prefix + "/include/qmodules.h.new", options.install_prefix + "/include/qmodules.h" ) ) {
        QFile::remove
            ( options.install_prefix + "/include/qmodules.h" );
        qtInstallPrefixDir.rename( options.install_prefix + "/include/qmodules.h.new", options.install_prefix + "/include/qmodules.h" );
        // TODO: check if original file is readonly, in this case the config isn't updated
    } else
        QFile::remove
            ( options.install_prefix + "/include/qmodules.h.new" );

    //#-------------------------------------------------------------------------------
    //# part of configuration information goes into qconfig.h
    //#-------------------------------------------------------------------------------

    f.setFileName( options.install_prefix + "/src/corelib/global/qconfig.h.new" );
    if ( f.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
        int i;

        QTextStream s( &f );
        s << "/* License information */\n";
		s << "#define QT_PRODUCT_LICENSEE \"Open Source\"\n";
		s << "#define QT_PRODUCT_LICENSE  \"OpenSourceDesktop\"\n";
        s << "\n";
        s << "/* Build key */\n";
        s << "#define QT_BUILD_KEY \"" << getenv( "QMAKESPEC" ) << "\"\n";
        s << "\n\n/* Machine byte-order */\n";
        s << "#define Q_BIG_ENDIAN 4321\n";
        s << "#define Q_LITTLE_ENDIAN 1234\n";
        s << "#define Q_BYTE_ORDER Q_LITTLE_ENDIAN\n";

        s << "\n/* Compile time features */\n";
        s << "#define QT_POINTER_SIZE " << sizeof( void* ) << "\n";
        for ( i = 0; i < options.QStylesNo.count(); ++i ) {
            s << "#ifndef QT_NO_STYLE_" << options.QStylesNo[ i ].toUpper() << "\n";
            s << "# define QT_NO_STYLE_" << options.QStylesNo[ i ].toUpper() << "\n";
            s << "#endif\n\n";
        }
        WriteUnusedImageformats( s, "png" );
        WriteUnusedImageformats( s, "gif" );
        WriteUnusedImageformats( s, "mng" );
        WriteUnusedImageformats( s, "jpeg" );
        for ( i = 0; i < options.QCFlags.count(); i++ ) {
            s << "#ifndef " << options.QCFlags[ i ].toUpper() << "\n";
            s << "# define " << options.QCFlags[ i ].toUpper() << "\n";
            s << "#endif\n\n";
        }
        s << "#ifndef QT_QWINEXPORT" << endl;
        s << "#define QT_QWINEXPORT" << endl;
        s << "#endif" << endl;
        if ( !options.QConfig.contains( "staticlib" ) ) {
            s << "#ifndef QT_DLL" << endl;
            s << "#define QT_DLL" << endl;
            s << "#endif" << endl;
        }
        f.close();
    }

    // Avoid rebuilds checking if it didn't change.
    if ( !sameFile( options.install_prefix + "/src/corelib/global/qconfig.h.new", options.install_prefix + "/src/corelib/global/qconfig.h" ) ) {
        QFile::remove
            ( options.install_prefix + "/src/corelib/global/qconfig.h" );
        qtInstallPrefixDir.rename( options.install_prefix + "/src/corelib/global/qconfig.h.new", options.install_prefix + "/src/corelib/global/qconfig.h" );
        /* When qconfig.h changed, we have to delete the plugins */
        deletePlugins( options.QT_INSTALL_PLUGINS );

    } 
	else
        QFile::remove( options.install_prefix + "/src/corelib/global/qconfig.h.new" );

	if ( !QFile::exists( options.install_prefix + "/include/QtCore/qconfig.h" ) )
	{
		out << "Creating include/QtCore/qconfig.h" << endl;
		f.setFileName( options.install_prefix + "/include/QtCore/qconfig.h" );
		if ( f.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
			QTextStream s( &f );
			s << "#include \"../../src/corelib/global/qconfig.h\"\n";
			f.close();
		}
	}

	out << "Creating .qmake.cache" << endl;
    f.setFileName ( options.install_prefix + "/.qmake.cache" );
	if ( f.open( QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate ) ) {
		QTextStream s( &f );
        QString QMAKE_OUTDIR;
        if ( options.QConfig.contains( "debug" ) )
            QMAKE_OUTDIR += "debug";
        else if ( options.QConfig.contains( "release" ) )
            QMAKE_OUTDIR += "release";
        if ( options.QConfig.contains( "shared" ) )
            QMAKE_OUTDIR += "-shared";
        else if ( options.QConfig.contains( "staticlib" ) )
            QMAKE_OUTDIR += "-static";
        if ( options.QConfig.contains( "thread" ) )
            QMAKE_OUTDIR += "-mt";

        s << "QMAKE_QT_VERSION_OVERRIDE=" << QString::number( ( QT_VERSION >> 16 ) & 0xff ) << endl;
        s << "OBJECTS_DIR = tmp/obj/" << QMAKE_OUTDIR << endl;
        s << "MOC_DIR = tmp/moc/" << QMAKE_OUTDIR << endl;
		if ( !options.QStyles.isEmpty() ) 
			s << "styles += " << options.QStyles.join( " " ) << endl;
		if ( !options.QImageFmtsPlugin.isEmpty() )
			s << "imageformat-plugins += " << options.QImageFmtsPlugin.join( " " ) << endl;
		s << "sql-drivers += " << options.sqlDriver.join( " " ) << endl;
		s << "sql-plugins += " << options.sqlDriverPlugin.join( " " ) << endl;
		s << "style-plugins += " << options.QStylesPlugin.join( " " ) << endl;
		s << "CONFIG += " << options.QConfig.join( " " ) << " "
			<< options.QModules.join( " " ) << " "
			<< options.QImageFmts.join( " " ) << endl;
		s << "QMAKESPEC=" << options.qmakespec << endl;
        s << "ARCH = windows\n";
		s << "QT_BUILD_TREE = " << options.install_prefix << endl;
		s << "QT_SOURCE_TREE = " << options.install_prefix << endl;
		s << "QT_INSTALL_PREFIX = " << options.install_prefix << endl;
        s << "docs.path = " << options.QT_INSTALL_DOCS << endl;
        s << "headers.path = " << options.QT_INSTALL_HEADERS << endl;
        s << "plugins.path = " << options.QT_INSTALL_PLUGINS << endl;
        s << "libs.path = " << options.QT_INSTALL_LIBS << endl;
		s << "translations.path = " << options.QT_INSTALL_TRANSLATIONS << endl;
		s << "examples.path = " << QDir::convertSeparators( options.install_prefix + "/examples" ) << endl;
		s << "demos.path = " << QDir::convertSeparators( options.install_prefix + "/demos" ) << endl;

        f.close();
    }

    out << "Building qmake" << endl;
    buildqmake();

	//#-------------------------------------------------------------------------------
	//# save configuration into qconfig.pri
	//#-------------------------------------------------------------------------------

	f.setFileName ( options.install_prefix + "/mkspecs/qconfig.pri" );
	if ( f.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
		/* only shared/static, thread, release/debug, stl, rtti and exceptions */
		QTextStream s( &f );
		s << "CONFIG += ";
		if ( options.QConfig.contains( "shared" ) )
			s << "shared ";
		else if ( options.QConfig.contains( "staticlib" ) )
			s << "static ";

		if ( options.QConfig.contains( "thread" ) )
			s << "thread ";

		if ( options.QConfig.contains( "debug" ) )
			s << "debug ";
		else if ( options.QConfig.contains( "release" ) )
			s << "release ";

		if ( options.QConfig.contains( "stl" ) )
			s << "stl ";

		if ( options.QConfig.contains( "rtti" ) )
			s << "rtti ";

		if ( options.QConfig.contains( "exceptions" ) )
			s << "exceptions ";

		s << endl;
		s << "QT_CONFIG += " << options.QConfig.join( " ") << endl;
		s << "QT_PRODUCT = OpenSourceDesktop" << endl;

		f.close();
	}

	//#-------------------------------------------------------------------------------
	//# build makefiles based on the configuration
	//#-------------------------------------------------------------------------------

    doBuild();
}

void usage( void )
{
    out << "Usage:  configure [-prefix dir] [-buildkey key] [-docdir dir] [-headerdir dir] " << endl;
    out << " [-libdir dir] [-bindir dir] [-plugindir dir ] [-datadir dir]                  " << endl;
    out << " [-translationdir dir] [-sysconfdir dir] [-debug] [-release]                   " << endl;
    out << " [-no-gif] [-qt-gif] [-no-sm] [-sm] [-qt-zlib] [-system-zlib]                  " << endl;
    out << " [-qt-libjpeg] [-system-libjpeg] [-qt-libpng] [-system-libpng]                 " << endl;
    out << " [-qt-libmng] [-system-libmng] [-no-thread] [-thread]                          " << endl;
    out << " [-no-largefile] [-largefile]                                                  " << endl;
    out << " [-no-stl] [-stl] [-no-ipv6] [-ipv6] [-Istring] [-lstring]                     " << endl;
    out << " [-Lstring] [-Rstring] [-disable-<module>] [-with-<module setting>]            " << endl;
    out << " [-without-<module setting>] [-fast] [-no-fast]                                " << endl;
    out << "                                                                               " << endl;
    out << "Installation options:                                                          " << endl;
    out << "                                                                               " << endl;
    out << " These are optional, but you may specify install directories.                  " << endl;
    out << "                                                                               " << endl;
    out << "    -prefix dir ........ This will install everything relative dir             " << endl;
    out << "     (default $QT_INSTALL_PREFIX)                                            " << endl;
    out << "                                                                               " << endl;
    out << " You may use these to separate different parts of the install:                 " << endl;
    out << "                                                                               " << endl;
    out << "    -bindir dir ........ Executables will be installed to dir                  " << endl;
    out << "     (default PREFIX/bin)                                                    " << endl;
    out << "    -libdir dir ........ Libraries will be installed to dir                    " << endl;
    out << "     (default PREFIX/lib)                                                    " << endl;
    out << "    -docdir dir ........ Documentation will be installed to dir                " << endl;
    out << "     (default PREFIX/doc)                                                    " << endl;
    out << "    -headerdir dir ..... Headers will be installed to dir                      " << endl;
    out << "     (default PREFIX/include)                                                " << endl;
    out << "    -plugindir dir ..... Plugins will be installed to dir                      " << endl;
    out << "                          (default PREFIX/plugins)                             " << endl;
    out << "    -datadir dir ....... Data used by Qt programs will be installed to dir     " << endl;
    out << "                          (default PREFIX)                                     " << endl;
    out << "    -translationdir dir  Translations of Qt programs will be installed to dir  " << endl;
    out << "                          (default PREFIX/translations)                        " << endl;
    out << "    -sysconfdir dir ...  Settings used by Qt programs will be looked for in dir" << endl;
    out << "                          (default PREFIX/etc/settings)                        " << endl;
    out << "                                                                               " << endl;
#if 0

    out << " You may use these options to turn on strict plugin loading.                   " << endl;
    out << "                                                                               " << endl;
    out << "    -buildkey key ...... Build the Qt library and plugins using the specified  " << endl;
    out << "    key.  When the library loads plugins, it will only                         " << endl;
    out << "    load those that have a matching key.                                       " << endl;
    out << "                                                                               " << endl;
#endif

    out << "Configure options:                                                             " << endl;
    out << "                                                                               " << endl;
    out << " The defaults (*) are usually acceptable.  If marked with a plus (+) a test    " << endl;
    out << " for that feature has not been done yet, but will be evaluated later, the      " << endl;
    out << " plus simply denotes the default value. Here is a short explanation of each    " << endl;
    out << " option:                                                                       " << endl;
    out << "                                                                               " << endl;
    out << " *  -incremental ....... Use incremental linking.                              " << endl;
    out << "    -no-incremental .... Don't use incremenatal linking.                       " << endl;
    out << "                                                                               " << endl;
    out << " *  -release ........... Compile and link Qt with debugging turned off.        " << endl;
    out << "    -debug ............. Compile and link Qt with debugging turned on.         " << endl;
    out << "                                                                               " << endl;
    out << " *  -shared ............ Create and use a shared Qt library (libqt.so).        " << endl;
    out << "    -static ............ Create and use a static Qt library (libqt.a).         " << endl;
    out << "                                                                               " << endl;
    out << "    -fast .............. Configure Qt quickly by generating Makefiles only for " << endl;
    out << "                         library and subdirectory targets.  All other Makefiles" << endl;
    out << "                         are created as wrappers, which will in turn run qmake." << endl;
    out << "" << endl;
    out << " *  -no-fast ........... Configure Qt normally by generating Makefiles for all " << endl;
    out << "                         project files.                                        " << endl;
    out << "                                                                               " << endl;
    out << "    -largefile ......... Use largefile support (64bit).                        " << endl;
    out << " *  -no-largefile ...... Don't use largefile support.                          " << endl;
    out << "                                                                               " << endl;
    out << " *  -thread ............ Compile with thread support.                          " << endl;
    out << "    -no-thread ......... Don't compile with thread support                     " << endl;
    out << "                                                                               " << endl;
    out << " *  -no-gif ............ Do not compile in GIF reading support.                " << endl;
    out << "    -gif    ............ Compile in GIF reading support.                       " << endl;
    out << "    See src/kernel/qgif.h                                                      " << endl;
    out << "                                                                               " << endl;
    out << "    -no-zlib ........... Do not compile in ZLIB support.                       " << endl;
    out << " *  -zlib    ........... Compile in ZLIB support.                              " << endl;
    // fprintf(out," *  -qt-zlib ........... Use the zlib bundled with Qt.                         \n");
    // fprintf(out,"    -system-zlib ....... Use zlib from the operating system.                   \n");
    // fprintf(out,"    See http://www.info-zip.org/pub/infozip/zlib                             \n");
    out << "" << endl;
    out << "    -platform target ... The operating system and compiler you are building on." << endl;
    out << "    See the PLATFORMS file for a list of supported                           " << endl;
    out << "    operating systems and compilers.                                         " << endl;
    out << "                                                                               " << endl;
    // fprintf(out,"    -Dstring ........... Add an explicit define to the preprocessor.           \n");
    // fprintf(out,"    -Istring ........... Add an explicit include path.                         \n");
    // fprintf(out,"    -Lstring ........... Add an explicit library path.                         \n");
    // fprintf(out,"    -Rstring ........... Add an explicit dynamic library runtime search path.  \n");
    // fprintf(out,"    -lstring ........... Add an explicit library.                              \n");
    out << "                                                                               " << endl;
    out << "    -disable-<module> .. Disable <module> where module is one of:           " << endl;
    out << "                            [ $CFG_MODULES_AVAILABLE ]                         " << endl;
    out << "                                                                               " << endl;
    if ( ifConfig( "sql" ) ) {
        out << "    -qt-sql-<driver> ... Enable a SQL <driver> in the Qt Library, by default " << endl;
        out << "                            none are turned on.                              " << endl;
        out << "    -plugin-sql-<driver> Enable SQL <driver> as a plugin to be linked to     " << endl;
        out << "                            at run time.                                     " << endl;
        out << "    -no-sql-<driver> ... Disable SQL <driver> entirely.                      " << endl;
        out << "                                                                             " << endl;
        out << "                            Possible values for <driver>:                    " << endl;
        out << QString("                            [ %1 ]").arg( options.sqlDriverAvailable.join( " " ) ) << endl;
        out << "                                                                             " << endl;
        out << "                         By default none of the SQL drivers are built,       " << endl;
        out << "                         even if they are auto-detected.                     " << endl;
        out << "                                                                             " << endl;
    }
    out << "    -qt-style-<style> .. Enable a GUI <style> in the Qt Library, by default " << endl;
    out << "                            all available styles are on.                       " << endl;
    out << "    -plugin-style-<style>Enable GUI <style> as a plugin to be linked to     " << endl;
    out << "                            at run time.                                       " << endl;
    out << "    -no-style-<style> .. Disable GUI <style> entirely.                      " << endl;
    out << "                                                                               " << endl;
    out << "                            Possible values for <style>:                       " << endl;
    out << QString("                            [ %1 ]").arg( options.QStyles.join( " " ) ) << endl;
    out << "                                                                               " << endl;
    out << "    -no-pch ............ Do not use precompiled header support.             " << endl;
    out << " *  -pch ............... Use precompiled header support.                    " << endl;
    out << "                                                                            " << endl;
    out << "    -no-stl ............ Do not compile STL support.                        " << endl;
    out << " *  -stl ............... Compile STL support.                               " << endl;
    out << "                                                                            " << endl;
    out << "    -no-rtti ........... Do not compile RTTI support.                       " << endl;
    out << " *  -rtti .............. Compile RTTI support.                              " << endl;
    out << "                                                                            " << endl;
    out << "    -no-exceptions ..... Enable exceptions on platforms that support it.    " << endl;
    out << "    -exceptions ........ Disable exceptions on platforms that support it.   " << endl;
    out << "                                                                            " << endl;
    out << "    -verbose ........... Print verbose information about each step of the   " << endl;
    out << "    -v ................. configure process.                                 " << endl;
    out << "                                                                            " << endl;
    out << "    -rebuild............ rebuild makefiles using the current configuration. " << endl;
    out << "                                                                            " << endl;
    out << "    -dryrun ............ don't execute real command only display them.      " << endl;
    out << "                                                                            " << endl;
    out << "    -no-dsp ............ Do not create Microsoft Visual C++ files.          " << endl;
    out << " *  -dsp    ............ Create Microsoft Visual C++ dsp files.             " << endl;
    out << "                                                                            " << endl;
    out << "    -make-files ........ create makefiles for a outside qt source package.  " << endl;
    out << "                                                                            " << endl;
    // fprintf(out,"    -continue........... Continue as far as possible if an error occurs.    \n");
    // fprintf(out,"                                                                               \n");
    out << "    -help .............. Print help and usage information.                  " << endl;
    out << "    -h ................. Print help and usage information.                  " << endl;
    out << "                                                                               " << endl;
    out << "Image Formats:                                                                 " << endl;
    out << "                                                                               " << endl;
    out << "    -plugin-imgfmt-<fmt> Enable <format> to be linked to at runtime.           " << endl;
    out << "                            Uses specified lib<format>.                        " << endl;
    out << "    -qt-imgfmt-<format>  Enable <format> to be linked into Qt.                 " << endl;
    out << "                            Uses specified lib<format>.                        " << endl;
    out << "    -no-imgfmt-<format>  Fully disable <format> from Qt.                        " << endl;
    out << "                                                                               " << endl;
    out << "                            Possible values for <format>:                      " << endl;
    out << "                            [ png jpeg mng ]                                   " << endl;
    out << "                                                                               " << endl;
    // fprintf(out,"Third Party Image Loading Libraries:                                           \n");
    // fprintf(out,"                                                                               \n");
    // fprintf(out," *  -qt-libpng ......... Use the libpng bundled with Qt.                       \n");
    // fprintf(out,"    -system-libpng ..... Use libpng from the operating system.                 \n");
    // fprintf(out,"    See http://www.libpng.org/pub/png                                        \n");
    // fprintf(out,"                                                                               \n");
    // fprintf(out," *  -qt-libjpeg ........ Use the libjpeg bundled with Qt.                      \n");
    // fprintf(out,"    -system-libjpeg .... Use jpeglib from the operating system.                \n");
    // fprintf(out,"    See http://www.ijg.org                                                   \n");
    // fprintf(out,"                                                                               \n");
    // fprintf(out," *  -qt-libmng ......... Use the libmng bundled with Qt.                       \n");
    // fprintf(out,"    -system-libmng ..... Use libmng from the operating system.                 \n");
    // fprintf(out,"    See http://www.libmng.com                                                \n");
}


int main( int argc, char **argv )
{
    QStringList commandLine;
    for ( int i = 0; i < argc; ++i ) {
        commandLine.append( argv[ i ] );
    }

    parseCommandLine( commandLine );

	QFile log;
    if ( !options.logfile.isEmpty() ) {
        log.setFileName( options.logfile );
		if ( !log.open( QIODevice::WriteOnly ) )
            out << "Error, could not open log file" << endl;
		else
			out.setDevice( &log );
    }

    if ( options.rebuild )
        doBuild();
    else if ( options.makeFiles )
        makeFiles();
    else
        doConfigure();

	return 0;
}
